package lm97xx;

use USB;
use Time::HiRes qw( usleep);



my $hdl;
my $dumb = "123456789";
my $len=0;


sub auswertung($$)
{
  my ($len,$msg)=@_;
  my @data;
  print "Len: $len ";
  if($len>0)
  {
    print ": "; #.substr($msg,0,$len)." : ";
    @data= unpack("C*",substr($msg,0,$len));
    printf("%02X ",$_) foreach (@data);
  }
  print "\n\n";
  return @data;
}

sub cmd
{
  my ($self,$len,$dumb)=($_[0],0,$_[1]);
  print "$_[2]:\n";
  $len=&USB::usb_control_msg($self->{hdl}, $_[3], $_[4], $_[5], 0x0000, $dumb, length($dumb), 500);
  return auswertung($len,$dumb);
}



sub new
{
  my ($class) = shift; $class = ref($class) if ref($class);

  &USB::usb_init();
  &USB::usb_find_busses();
  &USB::usb_find_devices();

  my($busses) = &USB::get_usb_busses();
  do
  {
    printf("Scanning bus name = %s\n",  &USB::get_usb_busname($busses));
    my($devices) = &USB::get_usb_devices($busses);
    do
    {
      printf("Scanning device name = %s\n",  &USB::get_usb_devicename($devices));
      %tt = %{ USB::get_usb_device_descriptor($devices) };

      if($tt{idVendor}==0x0400 and $tt{idProduct}==0x0fff)
      {
        foreach (keys(%tt))
        {
          #printf("key = '%s' data='0x%04X'\n", $_, $tt{$_});
          printf("%s: 0x%04X\n", $_, $tt{$_});
  
        }
        printf("idVendor=0x%04X idProduct =0x%04X\n", $tt{"idVendor"}, $tt{"idProduct"});

        my $self=bless {},$class;
 
        $self->{hdl} = &USB::usb_open($devices);

        print "Handle: $self->{hdl}\n";
        print "Devices: $devices\n";

        &USB::usb_set_debug(255);
        #&USB::usb_reset($self->{hdl}); # Scherz?
        &USB::usb_claim_interface($self->{hdl},0);
        &USB::usb_set_altinterface($self->{hdl},0);
        &USB::usb_set_configuration($self->{hdl},1);

        return $self; 


      }
    } while ($devices = &USB::get_usb_next_device($devices));
  } while ($busses = &USB::get_usb_next_bus($busses));
  return undef;
}



sub getVersion
{
  my @arr=@{cmd(shift,"    ","Version",0xC0,0x04,0x00D1)};
  return "$arr[1].$arr[0]";
}

sub getResolution
{
  return @{cmd(shift,"    ","GetResolution",0xC0,0x0C,0x00D7)}->[0];
}

sub getDeviceAddress
{
  return @{cmd(shift,"    ","USB Device Address",0xC0,0x0C,0x00D8)}->[0];
}

sub getSensorType
{
  return @{cmd(shift,"    ","GetSensorType",0xC0,0x0C,0x00D0)}->[0];
}

sub getCameraSpyDebugInfo
{
  print "Get Camera Spy Debuginfo:\n";
  $dumb=pack("C*",(0,0x70,0,0,0,0));

  cmd(shift,pack("C*",(0,0x70,0,0,0,0)),"Get Camera Spy Debuginfo",0x40,0x04,0x00FC);

  my $thistime=512; 
  my $buffer=" " x $thistime;
  $len = &USB::usb_bulk_read($hdl,0x85,$buffer,$thistime,500);
  auswertung($thistime,substr($buffer,0,$thistime));
  open OUT,">cameraspydebug.bin";
  print OUT substr($buffer,0,$thistime);
  close OUT;
}

sub getcapturequality
{
  # getcapturequality(1) ->     1% jpeg quality
  # getcapturequality(100) -> 100% jpeg quality

  my $ret=@{cmd(shift,pack("C*",(8,0,0,0,0,$_[0])),0x40,0x04,0x00FC)}->[0];
  usleep(100000);
  return $ret;
}

# Hier mu unbedingt ein bischen gewartet werden, einer der vorherigen Befehle bentigt offensichtlich Zeit.
# Bitte genauer durchtesten, welcher.

sub led
{
  cmd(shift,pack("C*",(0x02,0x90,0,0,0,3)),"LED",0x40,0x04,0xFC);
}

sub ff
{
  cmd(shift,"    ","FF",0xC0,0x0C,0xFF);
}

sub getBufferStatus
{
  return @{cmd(shift,"    ","Buffer Status",0xC0, 0x0C, 0x00E5)}->[0];
}

sub getImageSize
{
  my $arr=@{cmd(shift,"    ","ImageSize",0xC0, 0x0C, 0x00FD)};
  return $arr[2]*256*256+$arr[1]*256+$arr[0];
}


sub setResolution
{
  cmd(shift,pack("C*",(0x0010,0,0,0,0,1)),"SetResolution",0xC0,0x04,0x00FC);
}

sub flush
{
  cmd(shift,"    ","Flush",0x40,0x0C,0x00FA);
  usleep(100000);
}


sub getCommandStatus
{
  my $ret=@{cmd(shift,"","Command Status",0xC0,0x0C,0xE4)}->[0];
  usleep(100000);
  return $ret;
}


sub getImage
{
  my ($self,$ready,$len,$dumb,$image)=(shift,0,0,"","");
  my $hdl=$self->{hdl};

  $dumb=pack("C*",(0x00,0,0,0));
  while(defined($ready) and $ready ne 1)
  {
    print "Buffer Status:\n";
    $len=&USB::usb_control_msg($hdl, 0xC0, 0x0C, 0x00E5, 0x0000, $dumb, 0x0008, 1000);
    $ready=@{auswertung($len,$dumb)}[0];
  }

  print "Snap Start:\n";
  $dumb=pack("C*",(0x02));
  $len=&USB::usb_control_msg($hdl, 0x40, 0x0C, 0x00FB, 0x0000, $dumb, length($dumb), 1000);
  auswertung($len,$dumb);

  print "Image Size:\n";
  $dumb=pack("C*",(0x0010,0,0,0,0,1));
  $len=&USB::usb_control_msg($hdl, 0xC0, 0x04, 0x00FD, 0x0000, $dumb, length($dumb), 1000);
  my @imagesize_=auswertung($len,$dumb);

  my $imagesize=$imagesize_[1]*256+$imagesize_[0];

  print "Imagesize: $imagesize\n";

  while($imagesize>0)
  {
    #my $thistime=512; #($imagesize>4096) ? 4096 : 64;
    my $thistime=($imagesize>4096) ? 4096 : 64;
    #$thistime=$imagesize;

    my $buffer=" " x ($thistime+1);
    #print "Bereits geholt: ".length($image).", Dieses Mal: $thistime, noch brig: $imagesize\n";
    $len = &USB::usb_bulk_read($hdl,0x85,$buffer,$thistime,500);
    #print "Len: $len\n";
    $imagesize-=$thistime;  

    #print "Die Lnge des Bildes, das wir bekommen haben ist $len\n";
    #print "Noch $imagesize Bytes ...\n";

    $image.=substr($buffer,0,$thistime);
  }

  print "Die Gesamtlnge ist : ".length($image)."\n";

  print "Transfer Done:\n";
  $dumb=pack("C*",(1));
  $len=&USB::usb_control_msg($hdl, 0x40, 0x0C, 0x00FE, 0x0000, $dumb, 0x0008, 1000);
  auswertung($len,$dumb);

  return $image;
}

# Wenn als zweiter Parameter (direkt nach $hdl) 01 statt C0 geschickt wir, dann hngt
# sich die Kamera mit dem Multibyte Problem auf ... -> Kamera ausstecken, neu einstecken


if(1)
{
  system("rmmod usb-uhci");
  system("insmod usb-uhci");
  sleep 1;
}


1;



